@@ -1,3 +1,10 @@
+// Licensed to the .NET Foundation under one or more agreements.
+// The .NET Foundation licenses this file to you under the MIT license.
+
+#ifdef __cplusplus
+extern "C" {
+#endif // __cplusplus
+
 uint8_t
 dn_simdhash_string_ptr_try_add (dn_simdhash_string_ptr_t *hash, const char *key, void *value);
 
@@ -11,3 +18,7 @@ typedef void (*dn_simdhash_string_ptr_foreach_func) (const char *key, void *valu
 
 void
 dn_simdhash_string_ptr_foreach (dn_simdhash_string_ptr_t *hash, dn_simdhash_string_ptr_foreach_func func, void *user_data);
+
+#ifdef __cplusplus
+}
+#endif // __cplusplus
